# Generated by Django 4.2.16 on 2024-10-13 08:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("hope_smart_import", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="configuration",
            name="mapping",
            field=models.JSONField(blank=True, default=dict),
        ),
    ]



import os
import sys
from pathlib import Path
from typing import TYPE_CHECKING

import pytest
from django.contrib.auth.hashers import make_password

if TYPE_CHECKING:
    from django.contrib.auth.models import User

here = Path(__file__).parent
DEMOAPP_PATH = here / "demoapp"
sys.path.insert(0, str(here / "../src"))
sys.path.insert(0, str(DEMOAPP_PATH))


def pytest_configure(config):
    os.environ.update(DJANGO_SETTINGS_MODULE="demo.settings")

    import django

    django.setup()


@pytest.fixture()
def std_user(db) -> "User":
    from demo.factories import UserFactory

    return UserFactory(
        username="admin@example.com",
        is_staff=True,
        is_active=True,
        is_superuser=False,
        password=make_password("password"),
    )



# Generated by Django 4.2.16 on 2024-10-03 09:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Household",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100)),
                ("data", models.JSONField(blank=True, default=dict)),
            ],
        ),
        migrations.CreateModel(
            name="Individual",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("data", models.JSONField(blank=True, default=dict)),
                ("household", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="demo.household")),
            ],
        ),
    ]



import time

from celery import shared_task
from concurrency.exceptions import RecordModifiedError


@shared_task(bind=True)
def process_job(self, pk, version=None):
    from .models import Job

    job = Job.objects.get(pk=pk)

    if version and job.version != version:
        raise RecordModifiedError(f"Unexpected version {version}. It should be {job.version}", target=job)

    if job.op == "upper":
        job.name = job.name.upper()
    elif job.op == "progress":
        for i in range(1, job.value):
            self.update_state(state="PROGRESS", meta={"current": i})
            time.sleep(0.5)
    elif job.op == "loop":
        start = time.time()
        for i in range(job.value):
            time.sleep(1)
        elapsed = time.time() - start
        return {"loops": job.value, "elapsed": elapsed}

    elif job.op == "sleep":
        time.sleep(job.value)

    elif job.op == "raise":
        raise Exception(job.name)
    else:
        raise Exception("Unknown {op}")
    job.save()
    return job.name


@shared_task()
def echo(value):
    return value



from django.db import models
from hope_flex_fields.models import DataChecker


class Configuration(models.Model):
    name = models.CharField(max_length=100, unique=True)
    checker = models.ForeignKey(DataChecker, on_delete=models.CASCADE)
    mapping = models.JSONField(default=dict, blank=True, null=False)



# Generated by Django 4.2.16 on 2024-10-03 09:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("hope_flex_fields", "0002_fieldset_content_type"),
    ]

    operations = [
        migrations.CreateModel(
            name="Configuration",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100, unique=True)),
                (
                    "checker",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="hope_flex_fields.datachecker"),
                ),
            ],
        ),
    ]



# from smart_env import SmartEnv
#
# CONFIG = {
#     "CELERY_VISIBILITY_TIMEOUT": (
#         int,
#         60,
#         60,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#debug",
#     ),
#     "CELERY_TASK_ALWAYS_EAGER": (
#         bool,
#         True,
#         False,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#debug",
#     ),
#     "CELERY_TASK_EAGER_PROPAGATES": (
#         bool,
#         True,
#         False,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#debug",
#     ),
#     "DEBUG": (
#         bool,
#         False,
#         True,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#debug",
#     ),
#     "CELERY_BROKER_URL": (
#         str,
#         "redis://localhost:6379/0",
#         "",
#         True,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#DATABASES",
#     ),
#     "USE_TZ": (
#         bool,
#         True,
#         True,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#debug",
#     ),
#     "SECURE_SSL_REDIRECT": (
#         bool,
#         True,
#         False,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#SECURE_SSL_REDIRECT",
#     ),
#     "SESSION_COOKIE_SECURE": (
#         bool,
#         True,
#         False,
#         False,
#         "https://docs.djangoproject.com/en/5.1/ref/settings/#SESSION_COOKIE_SECURE",
#     ),
#     "SECRET_KEY": (str, "", "", True, "Django SECRET_KEY"),
# }
#
# env = SmartEnv(**CONFIG)



# from pathlib import Path
#
# import pytest
#
# from hope_smart_import.utils import import_simple_xls
#
#
# @pytest.fixture()
# def xls_simple() -> str:
#     return str((Path(__file__).parent / "data" / "simple1.xlsx").absolute())
#
#
# def test_read_xls(xls_simple: str) -> None:
#     for e in import_simple_xls(xls_simple):
#         assert list(e.keys()) == ["name", "last_name", "gender"]



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User


class AnyUserAuthBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        if username:
            user, __ = User.objects.update_or_create(
                username=username,
                defaults=dict(
                    is_staff=True,
                    is_active=True,
                    is_superuser=True,
                    email=f"{username}@demo.org",
                ),
            )
            return user



import csv
from typing import TYPE_CHECKING, Any, Iterable

import openpyxl

if TYPE_CHECKING:
    from openpyxl.workbook.workbook import Workbook
    from openpyxl.worksheet.worksheet import Worksheet

    from .types import MultiSheetResult, SheetResult


def _read_worksheet(sheet: "Worksheet", start_at: int, headers: bool) -> Iterable[dict[str, Any]]:
    rows = sheet.rows
    if headers:
        field_names = [str(cell.value) for cell in next(rows)]
        for i, row in enumerate(rows):
            if i < start_at:
                continue
            yield dict(zip(field_names, (str(cell.value or "") for cell in row)))  # pragma: no branch
    else:
        for i, row in enumerate(rows):
            field_names = [f"column{d + 1}" for d in range(len(row))]
            if i < start_at:
                continue
            yield dict(zip(field_names, (str(cell.value or "") for cell in row)))  # pragma: no branch


def open_xls(filepath: str, *, sheet_index: int = 0, start_at: int = 0, headers: bool = True) -> "SheetResult":
    wb: "Workbook" = openpyxl.load_workbook(filepath)
    sh: "Worksheet" = wb.worksheets[sheet_index]
    for i in _read_worksheet(sh, start_at=start_at, headers=headers):
        yield i


def open_xls_multi(
    filepath: str, sheets: list[int] = (0,), start_at: list[int] | int = 0, headers: list[bool] | bool = True
) -> "MultiSheetResult":
    wb: "Workbook" = openpyxl.load_workbook(filepath)
    for si in sheets:
        sh: "Worksheet" = wb.worksheets[si]
        start_at = start_at if isinstance(start_at, int) else start_at[si]
        headers = headers if isinstance(headers, bool) else headers[si]
        yield si, _read_worksheet(sh, start_at=start_at, headers=headers)


def open_csv(filepath: str, *, start_at: int = 0, headers: bool = False) -> "SheetResult":
    with open(filepath) as f:
        if headers:
            reader = csv.DictReader(f)
            for i, row in enumerate(reader):
                if i < start_at:
                    continue
                yield row
        else:
            reader = csv.reader(f)
            for i, row in enumerate(reader):
                if i < start_at:
                    continue
                field_names = [f"column{d + 1}" for d in range(len(row))]
                yield dict(zip(field_names, row))



# mypy: disable-error-code="no-untyped-def"
from pathlib import Path

import pytest
from django import forms
from hope_flex_fields.models import Fieldset
from hope_flex_fields.models.definition import FieldDefinition
from hope_flex_fields.models.flexfield import FlexField

from hope_smart_import.readers import open_xls
from hope_smart_import.shortcuts import validate_single


@pytest.fixture()
def xls_path() -> str:
    return str((Path(__file__).parent / "data" / "simple2.xlsx").absolute())


def test_sample_code(db, xls_path):
    fs, __ = Fieldset.objects.get_or_create(name="test.xlsx")

    charfield = FieldDefinition.objects.get(field_type=forms.CharField)
    choicefield = FieldDefinition.objects.get(field_type=forms.ChoiceField)

    FlexField.objects.get_or_create(name="name", fieldset=fs, field=charfield)
    FlexField.objects.get_or_create(name="last_name", fieldset=fs, field=charfield)
    FlexField.objects.get_or_create(
        name="gender", fieldset=fs, field=choicefield, attrs={"choices": [["M", "M"], ["F", "F"]]}
    )

    errors = validate_single(open_xls(xls_path), fs, fail_if_alien=True)
    assert errors == {3: {"gender": ["Select a valid choice. X is not one of the available choices."]}}



from admin_extra_buttons.mixins import ExtraButtonsMixin
from django.contrib import admin

from .models import Configuration


@admin.register(Configuration)
class ConfigurationAdmin(ExtraButtonsMixin, admin.ModelAdmin):
    pass
